/*
 * hfcl_mlpf.c
 * Copyright (C) 2007, Hitachi, Ltd.
 * Author(s): Yoshihiro Toyohara <yoshihiro.toyohara.qs@hitachi.com>
 */

char mlpf_rcsid[] = "$Id: hfcl_mlpf.c,v 1.3.2.10.2.1.6.3 2009/06/14 11:54:17 eteshima Exp $";

#include "hfcldd.h"
#include "hfclddcom.h"
#include "hfcl_detect.h"
#include "hfcl_hand_timer_trace.h"
#include "hfcl_top.h"
#include "hfcl_tbol.h"
#include "hfcl_mlpf.h"
#include "hfcl_ioctl.h"
#include "hfcl_strategy.h"
#include "hfcl_timer_recovery.h"
#include "hfcldd_conf.h"

/* MMIO-HG space address map */										/* @MLPF STR */
#define HFC_IOHGSPC_PARTSNUM_LEN        0x10
#define HFC_IOHGSPC_SYSREV_LEN          0x4
#define HFC_IOHGSPC_ADAPID_LEN          0x10
#define HFC_IOHGSPC_VPDAREA_LEN         0x80
#define HFC_IOHGSPC_INDAREA_LIEN        0x80
#define HFC_IOHGSPC_EFI_OP_TBL_LEN      0x100
const struct hg_map hfc_hg_map[1] = {
	/* MMIO-HG Ver=0100 */
	{
		{
	      /* MMIO-HG */     /* no : name		 			   size */
	      { 0x0000,         /* 0  :  HFC_IOHGSPC_ZERO			-   */
	        0x0000,         /* 1  :  HFC_IOHGSPC_LENGTH			4B  */
	        0x0004,         /* 2  :  HFC_IOHGSPC_VERSION		4B  */
	        0x0008,         /* 3  :  HFC_IOHGSPC_HYPSTATUS		4B  */
	        0x000c,         /* 4  :  HFC_IOHGSPC_CMNDREG		4B  */
	        0x0010,         /* 5  :  HFC_IOHGSPC_VFCWWPN0		4B  */
	        0x0014,         /* 6  :  HFC_IOHGSPC_VFCWWPN1		4B  */
	        0x0018,         /* 7  :  HFC_IOHGSPC_VFCWWNN0		4B  */
	        0x001c,         /* 8  :  HFC_IOHGSPC_VFCWWNN1		4B  */
	        0x0020,         /* 9  :  HFC_IOHGSPC_RID			4B  */
	        0x0024,         /* 10 :  HFC_IOHGSPC_MLPFDDVER		4B  */
	        0x0028,         /* 11 :  HFC_IOHGSPC_LPARSTATUS		4B  */
	        0x0000,         /* 12 :  */
	        0x0000,         /* 13 :  */
	        0x0000,         /* 14 :  */
	        0x0000,         /* 15 :  */
	        0x0000,         /* 16 :  */
	        0x0000,         /* 17 :  */
	        0x0000,         /* 18 :  */
	        0x0000,         /* 19 :  */
	        0x0030,         /* 20 :  HFC_IOHGSPC_INDACC0		4B  */
	        0x0034,         /* 21 :  HFC_IOHGSPC_INDACC1		4B  */
	        0x0038,         /* 22 :  HFC_IOHGSPC_INDACC2		4B  */
	        0x003c,         /* 23 :  HFC_IOHGSPC_INDACC3		4B  */
	        0x0040,         /* 24 :  HFC_IOHGSPC_PARTSNUM0		4B  */
	        0x0044,         /* 25 :  HFC_IOHGSPC_PARTSNUM1		4B  */
	        0x0048,         /* 26 :  HFC_IOHGSPC_PARTSNUM2		4B  */
	        0x004c,         /* 27 :  HFC_IOHGSPC_PARTSNUM3		4B  */
	        0x0050,         /* 28 :  HFC_IOHGSPC_SYSREV0		4B  */
	        0x0054,         /* 29 :  HFC_IOHGSPC_SYSREV1		4B  */
	        0x0058,         /* 30 :  HFC_IOHGSPC_SYSREV2		4B  */
	        0x005c,         /* 31 :  HFC_IOHGSPC_SYSREV3		4B  */
	        0x0060,         /* 32 :  HFC_IOHGSPC_ADAPID0		4B  */
	        0x0064,         /* 33 :  HFC_IOHGSPC_ADAPID1		4B  */
	        0x0068,         /* 34 :  HFC_IOHGSPC_ADAPID2		4B  */
	        0x006c,         /* 35 :  HFC_IOHGSPC_ADAPID3		4B  */
	        0x0000,         /* 36 :  */
	        0x0000,         /* 37 :  */
	        0x0000,         /* 38 :  */
	        0x0000,         /* 39 :  */
	        0x0080,         /* 40 :  HFC_IOHGSPC_VPDAREA		128B */
	        0x0100,         /* 41 :  HFC_IOHGSPC_INDAREA		128B */
	        0x0000,         /* 42 :  */
	        0x0000,         /* 43 :  */
	        0x0000,         /* 44 :  */
	        0x0000,         /* 45 :  */
	        0x0000,         /* 46 :  */
	        0x0000,         /* 47 :  */
	        0x0000,         /* 48 :  */
	        0x0000,         /* 49 :  */
	        0x01f0,         /* 50 :  HFC_IOHGSPC_EFI_INFO0		4B  */
	        0x01f4,         /* 51 :  HFC_IOHGSPC_EFI_INFO1		4B  */
	        0x01f8,         /* 52 :  HFC_IOHGSPC_EFI_INFO2		4B  */
	        0x01fc,         /* 53 :  HFC_IOHGSPC_EFI_INFO3		4B  */
	        0x0200,         /* 54 :  HFC_IOHGSPC_EFI_OP_TBL0	4B  */
	        0x0204,         /* 55 :  HFC_IOHGSPC_EFI_OP_TBL1	4B  */
	        0x0000         /* 56 :  */
	        }
	    }
    }
};																	/* @MLPF END */

/*--------------------------------------------------------------------------*/
/* name : hfc_version()                                                     */
/*--------------------------------------------------------------------------*/

/*
 * Function:    HFC_MMODE_CHECK_BASIC
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *
 * Returns:     
 *
 * Notes:       
 */
int HFC_MMODE_CHECK_BASIC(
	struct adap_info            *ap)
{
	if ( !(ap->mlpf_mode & HFC_MMODE_MLPF ) )
		return HFC_MMODE_CHECK_OK;
	else
		return HFC_MMODE_CHECK_ERROR;
}


/*
 * Function:    HFC_MMODE_CHECK_MLPF
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *
 * Returns:     
 *
 * Notes:       
 */
int HFC_MMODE_CHECK_MLPF(
	struct adap_info            *ap)
{
	if (ap->mlpf_mode & HFC_MMODE_MLPF )
		return HFC_MMODE_CHECK_OK;
	else
		return HFC_MMODE_CHECK_ERROR;
}


/*
 * Function:    HFC_MMODE_CHECK_SHADOW
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *
 * Returns:     
 *
 * Notes:       
 */
int HFC_MMODE_CHECK_SHADOW(
	struct adap_info            *ap)
{
	if (ap->mlpf_mode & HFC_MMODE_SHADOW )
		return HFC_MMODE_CHECK_OK;
	else
		return HFC_MMODE_CHECK_ERROR;
}


/*
 * Function:    HFC_MMODE_CHECK_REBOOT
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *
 * Returns:     
 *
 * Notes:       
 */
int HFC_MMODE_CHECK_REBOOT(
	struct adap_info            *ap)
{
	if (ap->mlpf_mode & HFC_MMODE_REBOOT )
		return HFC_MMODE_CHECK_OK;
	else
		return HFC_MMODE_CHECK_ERROR;
}


/*
 * Function:    HFC_MMODE_CHECK_SHARED
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *
 * Returns:     
 *
 * Notes:       
 */
int HFC_MMODE_CHECK_SHARED(
	struct adap_info            *ap)
{
	if ( (ap->mlpf_mode & HFC_MMODE_MLPF ) && ! (ap->mlpf_mode & HFC_MMODE_DEDICATE ) )
		return HFC_MMODE_CHECK_OK;
	else
		return HFC_MMODE_CHECK_ERROR;
}


/*
 * Function:    HFC_MMODE_CHECK_DEDICATE
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *
 * Returns:     
 *
 * Notes:       
 */
int HFC_MMODE_CHECK_DEDICATE(
	struct adap_info            *ap)
{
	if ( (ap->mlpf_mode & HFC_MMODE_MLPF ) && (ap->mlpf_mode & HFC_MMODE_DEDICATE ) )
		return HFC_MMODE_CHECK_OK;
	else
		return HFC_MMODE_CHECK_ERROR;
}

#define MIN_IOBASE_LEN          0x100


/*
 * Function:    hfc_mlpf_setup_lparmode
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *
 * Returns:     
 *
 * Notes:       
 */
int hfc_mlpf_setup_lparmode(
	struct adap_info            *ap)
{
	struct pci_dev *pdev	= NULL;
	uint                      flag4, len4;
	uint                      base4;
	unsigned long             hyper_status;
	uint                        rid_int;
	uint64_t                    mmio_hg;
	
	if(HFC_MMODE_CHECK_SHADOW(ap)){
		HFC_DBGPRT("This LPAR is shadow LPAR\n");
	}
	else
		HFC_DBGPRT("This LPAR is Guest LPAR\n");
	
	/* PCI memory space mapping */
	pdev = ap->pci_cfginf;
	
	base4 = pci_resource_start(pdev, 4);
	len4 = pci_resource_len(pdev, 4);
	flag4 = pci_resource_flags(pdev, 4);
	
	if ( base4 == 0 )
	{
		ap->mlpf_mode &= ~( HFC_MMODE_MLPF | HFC_MMODE_DEDICATE );
		HFC_DBGPRT("hfc_mlpf_setup_lparmode: MMIO HG pointer NULL\n");
		if( HFC_MMODE_CHECK_SHADOW(ap)){
			HFC_DBGPRT("Shadow LPAR\n");
			HFC_ERRPRT("hfcldd : HFC_ERR9 FC Adapter Driver error (ErrNo:0xA3) \n"); 	/* FCLNX-0357 */
			HFC_ERRPRT("hfcldd : Invalid MMIO-HG Address.\n"); 							/* FCLNX-0357 */
			return HFC_MLPF_DISABLE;                                             /* FCLNX-0*** */
		}
		else
		{
			HFC_DBGPRT("Guest LPAR\n");
			return HFC_MLPF_ENABLE;  // When the setting is not invalid, it is ended with enable
		}
	}
	else
	{
		mmio_hg = (ulong)ioremap(base4, len4);  /* FCLNX-0352 */
		
		if (!(flag4 & IORESOURCE_MEM)) {
			if( ap->raslog_install )
				hfc_errlog(ap, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0xA3, NULL, 0) ;			/* FCLNX-0357 */
			else
				hfc_errlog(ap, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0xA3, NULL, 0);  	/* FCLNX-0357 */
			HFC_DBGPRT("  scsi(%ld): region #0 not a PIO resource, aborting\n",
				ap->host_no);
			HFC_DBGPRT("hfcldd :  Failed to pci_resource_flags \n");			 
			return HFC_MLPF_DISABLE;
		}
		
		if (len4 < MIN_IOBASE_LEN) {
			if( ap->raslog_install )
				hfc_errlog(ap, NULL, NULL, HFC_ERRLOG_TYPE_NONE, ERRID_HFCP_ERR9, 0xA3, NULL, 0) ;			/* FCLNX-0357 */
			else
				hfc_errlog(ap, NULL, NULL, HFC_ERRLOG_TYPE_RASLOG_EXIST, ERRID_HFCP_ERR9, 0xA3, NULL, 0);  	/* FCLNX-0357 */
			HFC_DBGPRT("hfcldd : Invalid MMIO-HG Length.\n"); 							/* FCLNX-0357 */
			HFC_DBGPRT("  scsi(%d): Invalid PCI I/O region size, aborting\n",
					(uint)ap->host_no);
			HFC_DBGPRT("hfcldd :  Failed to pci_resource_flags \n");			 
			return HFC_MLPF_DISABLE;
		}
	
		HFC_DBGPRT("mmio_hg NOT NULL\n");
		ap->hg_mem_base_addr = mmio_hg;
		HFC_DBGPRT("set ap->hg_mem_base_addr mmio_hg\n");
		
		ap->mlpf_mode |= HFC_MMODE_MLPF;
//		ap->hg_map  = (struct hg_map *) &hfc_hg_map[0]; /* FCLNX-0326 *//* FCLNX-0359 */

		ap->lparmode.hg_map = (struct hg_map *) &hfc_hg_map[0];         /* FCLNX-0374 */
		
		if(HFC_MMODE_CHECK_SHADOW(ap))
		{
			if ( hfc_mlpf_check_state(ap, HFC_HG_HYPSTATUS_REBOOT, HFC_CHECK_HYPER_STATE) )
			{
				HFC_DBGPRT("Shadow LPAR is rebooted\n");
				ap->mlpf_mode |= HFC_MMODE_REBOOT;
				hfc_mlpf_change_state(ap, HFC_HG_HYPSTATUS_REBOOT, HFC_DISABLE_LPAR_STATE);
			}
		}
		/* Refer MMIO-HG HyperStatus */
		HFC_DBGPRT("Refer MMIO-HG HyperStatus\n");
		hyper_status = (unsigned long) hfc_read_hg_reg(ap, HFC_IOHGSPC_HYPSTATUS, 0x4 );

		if (hyper_status & HFC_HG_HYPSTATUS_MODE)
		{
			/* dedicate mode */
			if(HFC_MMODE_CHECK_SHADOW(ap))
			{
				HFC_DBGPRT("hfc_mlpf_setup_lparmode: This Shadow LPAR is dedicate %x\n", ap->mlpf_mode);
				HFC_ERRPRT("hfcldd : HFC_ERR9 FC Adapter Driver error (ErrNo:0xA2) \n"); 	/* FCLNX-0357 */
				HFC_ERRPRT("hfcldd : Invalid Hyper Status Mode.\n"); 						/* FCLNX-0357 */
				hfc_mlpf_change_state(ap, HFC_HG_LPRSTATUS_ISVALID, HFC_ENABLE_LPAR_STATE);
				hfc_mlpf_change_state(ap, HFC_HG_LPRDETAIL_SPACE, HFC_DISABLE_LPAR_STATE);
				hfc_mlpf_change_state(ap, HFC_HG_LPRDETAIL_WWN_INVALID, HFC_ENABLE_LPAR_STATE);
				return HFC_MLPF_DISABLE;
			}
			
			ap->mlpf_mode |= HFC_MMODE_DEDICATE;
			
			HFC_DBGPRT("hfc_mlpf_setup_lparmode: This LPAR is dedicate %x\n", ap->mlpf_mode);
		} else 
		{
			/* shared mode */
			if (hyper_status & HFC_HG_HYPSTATUS_FDISABLE)
			{
				HFC_DBGPRT("hfc_mlpf_setup_lparmode: This LPAR is force disable %x\n", ap->mlpf_mode);
				return HFC_MLPF_DISABLE;
			}
			HFC_DBGPRT("hfc_mlpf_setup_lparmode: This LPAR is shared %x\n", ap->mlpf_mode);
		}
	}
	
	if(HFC_MMODE_CHECK_SHADOW(ap))
	{
		uchar                   adap_id[16];
		uchar                   sys_buf[4];
		
		HFC_DBGPRT("hfc_mlpf_setup_lparmode: Shadow LPAR set adap_id, sysrev %x\n", ap->mlpf_mode);
		
		// Sets adap_id and sys_buf to mmio-hg, but parts_number is set after.
		hfc_read_flash(ap, 0x10000, 16, adap_id);
		hfc_mlpf_set_mmio_hg(ap, adap_id, HFC_IOHGSPC_ADAPID0, HFC_IOHGSPC_ADAPID_LEN);
		
		hfc_read_flash(ap, 0, 4, sys_buf);
		hfc_mlpf_set_mmio_hg(ap, sys_buf, HFC_IOHGSPC_SYSREV0, HFC_IOHGSPC_SYSREV_LEN);
	}
	
	//Sets RID.
	if ( HFC_MMODE_CHECK_SHARED(ap) )
	{
		HFC_DBGPRT("hfc_mlpf_setup_lparmode: Shared LPAR set RID %x\n", ap->mlpf_mode);
		rid_int = (uint) hfc_read_hg_reg(ap, HFC_IOHGSPC_RID, 0x4 );
		ap->rid = rid_int;
		HFC_DBGPRT("hfc_mlpf_setup_lparmode: read RID from MMIOHG%x\n", ap->rid);
	}
	
	
	HFC_DBGPRT("hfc_mlpf_setup_lparmode: Exit HFC_MLPF_ENABLE %x\n", ap->mlpf_mode);
	return HFC_MLPF_ENABLE;
}


/*
 * Function:    hfc_mlpf_setup_wwn
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *
 * Returns:     
 *
 * Notes:       
 */
uint hfc_mlpf_setup_wwn(
	struct adap_info            *ap)
{

	uint                        wk;
	uchar                       adap_id[16];


	wk = (uint) hfc_read_hg_reg(ap, HFC_IOHGSPC_VFCWWPN0, 0x4 );
	ap->vfc_ww_name = wk;
	ap->vfc_ww_name <<= 32;
	HFC_DBGPRT("get vfc_wwpn form MMIOHG %llx\n", (unsigned long long)ap->vfc_ww_name);
	
	wk = (uint) hfc_read_hg_reg(ap, HFC_IOHGSPC_VFCWWPN1, 0x4 );
	ap->vfc_ww_name |= wk;

	wk = (uint) hfc_read_hg_reg(ap, HFC_IOHGSPC_VFCWWNN0, 0x4 );
	ap->vfc_node_name = wk;
	ap->vfc_node_name <<= 32;
	HFC_DBGPRT("get vfc_wwnn form MMIOHG %llx\n", (unsigned long long)ap->vfc_node_name);

	wk = (uint) hfc_read_hg_reg(ap, HFC_IOHGSPC_VFCWWNN1, 0x4 );
	ap->vfc_node_name |= wk;

	ap->ww_name = ap->vfc_ww_name;
	ap->node_name = ap->vfc_node_name;
	HFC_DBGPRT("set adap_info : wwpn %llx\n", (unsigned long long)ap->ww_name);
	HFC_DBGPRT("set adap_info : wwnn %llx\n", (unsigned long long)ap->node_name);
	
	hfc_mlpf_get_mmio_hg(ap, adap_id, HFC_IOHGSPC_ADAPID0, HFC_IOHGSPC_ADAPID_LEN);
	
	HFC_4B_TO_4L(wk, (*(uint*)(&adap_id[0])));
	ap->org_ww_name = wk;
	ap->org_ww_name <<= 32;
	HFC_4B_TO_4L(wk, (*(uint*)(&adap_id[4])));
	ap->org_ww_name |= wk;
	HFC_4B_TO_4L(wk, (*(uint*)(&adap_id[8])));
	ap->org_node_name = wk;
	ap->org_node_name <<= 32;
	HFC_4B_TO_4L(wk, (*(uint*)(&adap_id[12])));
	ap->org_node_name |= wk;
	
	ap->org_ww_name += ap->port_no * 2;
	ap->org_node_name += ap->port_no * 2;
	
	return HFC_MLPF_VFC_VALID;
}


/*
 * Function:    hfc_mlpf_set_mmio_hg
 *
 * Purpose:     This routine sets hfcldd_data from mp_adap_info or adap-info to MMIOHG.
 *              This routine is considered for Shadow LPAR.
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_mlpf_set_mmio_hg(
	struct adap_info            *ap,
	uchar                       *buf,
	uint                        regno,
	uint                        length)
{
	uchar                       *ptr;
	uint                        value;							/* FCLNX_GPL-0151 */
	uint                        i;
	uint                        offset;
	
	offset = ap->lparmode.hg_map->iosp.reg[regno];                  /* FCLNX-0330 *//* FCLNX-0374 */
	HFC_DBGPRT("hfc_mlpf_set_mmio_hg: offset %x\n", offset);
	HFC_DBGPRT("hfc_mlpf_set_mmio_hg: length %x\n", length);
	
	ptr = (uchar *)buf;
	
	for ( i = 0 ; i < (length/4) ; i++)                         /* FCLNX-0331 */
	{
		value = 0;
		value += 0xff000000 & ((uint)ptr[(i*4)] << 24);      /* FCLNX-0333 */
		value += 0x00ff0000 & ((uint)ptr[((i*4)+1)] << 16 );
		value += 0x0000ff00 & ((uint)ptr[((i*4)+2)] << 8 );
		value += 0x000000ff & (uint)ptr[((i*4)+3)];
		
		HFC_DBGPRT("hfc_mlpf_set_mmio_hg: value[%d] : %x\n", i, value);
		hfc_write_reg_hg_ext(ap, offset + (i*4), 4, value);
	}	
	
	return;
}


/*
 * Function:    hfc_mlpf_set_mmio_hg
 *
 * Purpose:     This routine gets VPDDATA from MMIOHG, and sets it mp_adap_info.
 *              This routine is considered for Guest LPAR.
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *  buf        -
 *  regno      -
 *  length     -
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_mlpf_get_mmio_hg(
	struct adap_info            *ap,
	uchar                       *buf,
	uint                        regno,
	uint                        length)
{
	uchar                       *ptr;
	uint                        value;
	uint                        i;
	uint                        offset;
	
	offset = ap->lparmode.hg_map->iosp.reg[regno];                  /* FCLNX-0374 */
	ptr = (uchar *)buf;
	
	for ( i = 0 ; i < (length / 4) ; i ++)          /* FCLNX-0332 */
	{
		value = 0;
		value = (uint)hfc_read_reg_hg_ext(ap, offset + (i*4), 4);
		
		/* store data with reverse order */
		*ptr++ = (uchar)((value >> 24) & 0xFF);
		*ptr++ = (uchar)((value >> 16) & 0xFF);
		*ptr++ = (uchar)((value >> 8) & 0xFF);
		*ptr++ = (uchar)(value & 0xFF);
	}	
	
	return;
}


/*
 * Function:    hfc_mlpf_config_check
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *
 * Returns:     
 *
 * Notes:       
 */
int hfc_mlpf_config_check(
	struct adap_info            *ap)
{
	uchar                       wk1;
	uchar                       wk2;
	uchar                       wk3;
	
	// This case is considered for Guest LPAR.
	// Guest LPAR doesn't have to check status about Link Initialize resoponse.
	// So, This routine is finished for Guest LPAR at success .
	if (!HFC_MMODE_CHECK_SHADOW(ap))
		return 0;
	
	wk1 = hfc_read_val( ap->fw_init_p->fw_iocinfo.flag);
	wk2 = hfc_read_val( ap->fw_init_p->fw_iocinfo.connect_type);
	wk3 = hfc_read_val( ap->fw_init_p->fw_iocinfo.npiv_valid);
	
	if ( !(wk1 & HFC_FABRIC_VALID) )
	{
		hfc_mlpf_change_state(ap, HFC_HG_LPRSTATUS_UNSHARABLE, HFC_ENABLE_LPAR_STATE);
		hfc_mlpf_change_state(ap, (HFC_HG_LPRDETAIL_SPACE | HFC_HG_LPRSTATUS_LINKDOWN), HFC_DISABLE_LPAR_STATE);	/* FCLNX-0396 */
		hfc_mlpf_change_state(ap, HFC_HG_LPRDETAIL_PRIVATE, HFC_ENABLE_LPAR_STATE);
		
		return HFC_MLPF_CONFIG_CHECK_ERROR;
	} else if ( wk2 == HFC_AL)
	{
		hfc_mlpf_change_state(ap, HFC_HG_LPRSTATUS_UNSHARABLE, HFC_ENABLE_LPAR_STATE);
		hfc_mlpf_change_state(ap, (HFC_HG_LPRDETAIL_SPACE | HFC_HG_LPRSTATUS_LINKDOWN), HFC_DISABLE_LPAR_STATE);	/* FCLNX-0396 */
		hfc_mlpf_change_state(ap, HFC_HG_LPRDETAIL_FCAL, HFC_ENABLE_LPAR_STATE);
		
		return HFC_MLPF_CONFIG_CHECK_ERROR;
	} else if ( !(wk3 & HFC_NPIV_VALID) )
	{
		hfc_mlpf_change_state(ap, HFC_HG_LPRSTATUS_UNSHARABLE, HFC_ENABLE_LPAR_STATE);
		hfc_mlpf_change_state(ap, (HFC_HG_LPRDETAIL_SPACE | HFC_HG_LPRSTATUS_LINKDOWN), HFC_DISABLE_LPAR_STATE);	/* FCLNX-0396 */
		hfc_mlpf_change_state(ap, HFC_HG_LPRDETAIL_NO_NPIV, HFC_ENABLE_LPAR_STATE);
		
		return HFC_MLPF_CONFIG_CHECK_ERROR;
	}
	
	hfc_mlpf_change_state(
		ap, 
		HFC_HG_LPRSTATUS_LINKDOWN | HFC_HG_LPRSTATUS_UNSHARABLE | HFC_HG_LPRDETAIL_SPACE, 
		HFC_DISABLE_LPAR_STATE);
	
	return HFC_MLPF_CONFIG_CHECK_OK;
}


/*
 * Function:    hfc_mlpf_check_state
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *  status     - 
 *  type       - 
 *
 * Returns:     
 *
 * Notes:       
 */
int hfc_mlpf_check_state(
	struct adap_info            *ap,
	uint                        status,
	uint                        type)
{
	uint                        wk4 = 0;
	uint                        check;
	
	HFC_DBGPRT("hfc_mlpf_check_state Enter\n");
	if ( type == HFC_CHECK_HYPER_STATE )
	{
		wk4 = hfc_read_hg_reg(ap, HFC_IOHGSPC_HYPSTATUS, 0x4);
	} else if ( type == HFC_CHECK_LPAR_STATE )
	{
		wk4 = hfc_read_hg_reg(ap, HFC_IOHGSPC_LPARSTATUS, 0x4);
	}
	
	check = wk4 & status;
	
	if ( check == status )
	{
		HFC_DBGPRT("hfc_mlpf_check_state End return 1\n");
		return 1;
	}
	else
	{
		HFC_DBGPRT("hfc_mlpf_check_state End return 0\n");
		return 0;
	}
}


/*
 * Function:    hfc_mlpf_change_state
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *  status     - 
 *  type       - 
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_mlpf_change_state(
	struct adap_info            *ap,
	uint                        status,
	uint                        type)
{
	uint                        wk4;
	
	if( type == HFC_ENABLE_HYPER_STATE )
	{
		wk4 = (uint)hfc_read_hg_reg(ap, HFC_IOHGSPC_HYPSTATUS, 0x4);
		wk4 |= status;
		hfc_write_hg_reg(ap, HFC_IOHGSPC_HYPSTATUS, 0x4, wk4 );
	} else if ( type == HFC_DISABLE_HYPER_STATE )
	{
		wk4 = (uint)hfc_read_hg_reg(ap, HFC_IOHGSPC_HYPSTATUS, 0x4);
		wk4 &= ~status;
		hfc_write_hg_reg(ap, HFC_IOHGSPC_HYPSTATUS, 0x4, wk4 );
	} else if ( type == HFC_ENABLE_LPAR_STATE )
	{
		wk4 = (uint)hfc_read_hg_reg(ap, HFC_IOHGSPC_LPARSTATUS, 0x4);
		wk4 |= status;
		hfc_write_hg_reg(ap, HFC_IOHGSPC_LPARSTATUS, 0x4, wk4 );
	} else if ( type == HFC_DISABLE_LPAR_STATE )
	{
		wk4 = (uint)hfc_read_hg_reg(ap, HFC_IOHGSPC_LPARSTATUS, 0x4);
		wk4 &= ~status;
		hfc_write_hg_reg(ap, HFC_IOHGSPC_LPARSTATUS, 0x4, wk4 );
	}
	
	return;
}


/*
 * Function:    hfc_read_reg_hg_ext
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *  offset     - 
 *  reg_size   - 
 *
 * Returns:     
 *
 * Notes:       
 */
uint64_t hfc_read_reg_hg_ext(
	struct adap_info            *ap,
	uint                        offset,
	char                        reg_size)
{
	uchar *ptr;
	ushort data16=0,rtn16=0;	/* FCLNX-0659 */
	uint   data32=0,rtn=0;
	
	if ( offset % reg_size ) {
		return (0);
	}

	ptr = (uchar *)(ap->hg_mem_base_addr + (ulong)offset);

	switch (reg_size) {
		case 1: 
				rtn = readb( (uchar  *) ptr );
				break;

		case 2: 
				data16 = readw( (ushort *) ptr );
				rtn16 = data16;	 					/* FCLNX-0659 *//* Endian Convert Error */
				HFC_2B_TO_2L(rtn16, data16);		/* FCLNX-0659 */
				rtn = (uint)rtn16;					/* FCLNX-0659 */
				break;

		case 4: 
				data32 = readl( (uint   *) ptr );
				rtn = data32;
				HFC_4B_TO_4L(rtn, data32);
				break;

		default: 
				break;
	}

	
	HFC_DBGPRT("hfc_read_reg_hg_ext: rtn %x\n", rtn);
	return(rtn);
}


/*
 * Function:    hfc_write_reg_hg_ext
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *  offset     - 
 *  reg_size   - 
 *  data       - 
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_write_reg_hg_ext(
	struct adap_info            *ap,
	uint                        offset,
	char                        reg_size,
	uint64_t                    data)
{
	uchar   *ptr;
	ushort  data16_1,data16_2=0;
	uint    data32_1,data32_2=0;

	if ( offset % reg_size ) {
		return;
	}
	
	ptr = (uchar *)( ap->hg_mem_base_addr + (ulong)offset);
	
	switch (reg_size) {
		case 1: 
				writeb( (unsigned)data, (uchar  *) ptr ); 
				break;
		case 2: 
				data16_1 = (ushort) data;
				data16_2 = data16_1;
				HFC_2B_TO_2L(data16_2, data16_1);
				writew( (unsigned)data16_2, (uchar  *) ptr );
				break;
		case 4: 
				data32_1 = (uint) data;
				data32_2 = data32_1;
				HFC_4B_TO_4L(data32_2, data32_1);

				writel( (unsigned)data32_2, (uchar  *) ptr );
				break;

		default: 
				break;
	}
	
	HFC_DBGPRT("hfc_write_reg_hg_ext end \n");
	return;
}


/*
 * Function:    hfc_mlpf_pci_error
 *
 * Purpose:     It happens as follows, the following functions are called with 
 *              fc_abend() for Guest LPAR, and it comes off
 *              case HFC_ABEND_SERR:
 *              case HFC_ABEND_PERR:
 *              case HFC_ABEND_SPERR:
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *  type       - 
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_mlpf_pci_error(
	struct adap_info            *ap,
	uchar                       type)
{
	uint                    err_no=0 ;
	uchar                   data[16];
	
	memset(data, 0, 16);
	
	if( type == HFC_ABEND_SERR )
		err_no = 0x00000032 ;
	
	if( type == HFC_ABEND_PERR )
		err_no = 0x00000033 ;
	
	if( type == HFC_ABEND_SPERR )
		err_no = 0x00000034 ;
	
	hfc_errlog(ap,NULL,NULL,HFC_ERRLOG_TYPE_MCK,ERRID_HFCP_ERRD,err_no,data,16) ;
	
	ap->pci_err_cnt++ ;
	
	return;
}


/*
 * Function:    hfc_mlpf_mck_recovery
 *
 * Purpose:     It is not in hfc_mck_recovery_five at mck and calls it as follows for shadow
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *  type       - 
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_mlpf_mck_recovery(
	struct adap_info            *ap,
	uchar                        type)
{
	hfc_mck_recovery_five(ap,type);
	
	hfc_write_reg(ap, ( uint )HFC_IOSPACE_INTA_MSK,( char )0x4, HFC_MLPF_REC_END );
	
	set_bit(HFC_MLPF_WAIT_MCKEND, (ulong *)&ap->status);
	
}


/*
 * Function:    hfc_mlpf_intr
 *
 * Purpose:     The following functions are called at the time of sharing after int_a bit[0 to 3] is checked
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *  int_a_reg  - 
 *
 * Returns:     
 *
 * Notes:       
 */
uchar hfc_mlpf_intr(
	struct adap_info            *ap,
	uint                        int_a_reg)
{
	uint                        int_a_rst;
	uint                        hyp_status;
	uint                        after_int_a;
	
	if( int_a_reg & HFC_MLPF_HWERR )
	{
		HFC_DBGPRT("hfcldd : hfcl_intr - HFC_MLPF_HWERR bit 23 occured\n");
		
		/* read HYPER status before reset int_a_reg.     */
		hyp_status = hfc_read_hg_reg(ap, HFC_IOHGSPC_HYPSTATUS, 0x4);
		
		/* reset PCI space for HW_MLPF_HWERR			 */
		int_a_rst = int_a_reg & HFC_MLPF_HWERR;
		hfc_write_reg(ap, ( uint )HFC_IOSPACE_INTA_RST,( char )0x4, int_a_rst);

		/* dummy read */
		after_int_a = (uint)hfc_read_reg(ap, (uint)HFC_IOSPACE_INTA, (char)0x4);
		hfc_mlpf_hwerr_int(ap, hyp_status);
		
		return(1);
	} else if( ( int_a_reg & HFC_MLPF_REC_END ) && (HFC_MMODE_CHECK_SHADOW(ap) ) )
	{                                              // Guest LPAR is not set this bit
		HFC_DBGPRT("hfcldd : hfcl_intr - HFC_MLPF_HWERR bit 22 occured\n");
		
		/* reset PCI space for HW_MLPF_HWERR			 */
		int_a_rst = int_a_reg & HFC_MLPF_REC_END;
		hfc_write_reg(ap, ( uint )HFC_IOSPACE_INTA_RST,( char )0x4, int_a_rst);

		/* dummy read */
		after_int_a = (uint)hfc_read_reg(ap, (uint)HFC_IOSPACE_INTA, (char)0x4);
		hfc_mlpf_mckend_int(ap);
		
		return(1);
	}
	
	return(0);
}


/*
 * Function:    hfc_mlpf_hwerr_int
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *  hyp_status - 
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_mlpf_hwerr_int(
	struct adap_info     *ap,
	unsigned int         hyp_status)
{
	HFC_DBGPRT(" hfcldd%d : hfc_mlpf_hwerr_int \n",ap->dev_minor);
	
	if( HFC_MMODE_CHECK_SHADOW(ap) )
	{
		// check Force CSTP INT
		if( hyp_status & ( HFC_HG_HYPSTATUS_FCSTP | HFC_HG_HYPSTATUS_FCSTP_IML ) )
		{
			hfc_mlpf_change_state(
				ap, 
				( HFC_HG_HYPSTATUS_FCSTP | HFC_HG_HYPSTATUS_FCSTP_IML ),
				HFC_DISABLE_HYPER_STATE );
			hfc_w_stop(ap, HFC_MLPF_FCSTP_TMR) ;
			clear_bit(HFC_MLPF_WAIT_FCSTP, (ulong *)&ap->status );
			
			if(  hyp_status &  HFC_HG_HYPSTATUS_FCSTP )
				hfc_mlpf_fcstp_int(ap, HFC_ABEND_FCSTP);
			else if ( hyp_status & HFC_HG_HYPSTATUS_FCSTP_IML )
				hfc_mlpf_fcstp_int(ap, HFC_ABEND_FCSTP_IML);
			
			return;
		}
		// check FMCK INT
		else if( hyp_status & HFC_HG_HYPSTATUS_FMCK )
		{
			hfc_mlpf_change_state(ap, HFC_HG_HYPSTATUS_FMCK, HFC_DISABLE_HYPER_STATE );
			hfc_w_stop(ap, HFC_MLPF_FMCK_TMR) ;
			clear_bit(HFC_MLPF_WAIT_FMCK, (ulong *)&ap->status );
				
			hfc_issue_forced_mck(ap, HFC_ABEND_T3);
			
			return;
		}
	}
	if( HFC_MMODE_CHECK_SHARED(ap) && !( HFC_MMODE_CHECK_SHADOW(ap) ) )
	{
		// check CSTP END
		if( hyp_status & HFC_HG_HYPSTATUS_CSTPEND )
		{
			hfc_mlpf_change_state(ap, HFC_HG_HYPSTATUS_CSTPEND, HFC_DISABLE_HYPER_STATE );
			hfc_w_stop(ap, HFC_MLPF_MCKEND_TMR) ;
			clear_bit(HFC_MLPF_WAIT_MCKEND, (ulong *)&ap->status );
			hfc_mlpf_cstpend_int(ap);
			
			return;
		}
		// check MCK INT(Hyper)
		else if(hyp_status & HFC_HG_HYPSTATUS_MCK )
		{
			if( test_bit( HFC_WAIT_T3, (ulong *)&ap->status ) )
			{
				clear_bit( HFC_WAIT_T3, (ulong *)&ap->status );
			}
			clear_bit(HFC_MLPF_WAIT_FMCK, (ulong *)&ap->status );
			hfc_mlpf_change_state(ap, HFC_HG_HYPSTATUS_MCK, HFC_DISABLE_HYPER_STATE );
			set_bit( HFC_MCK_RECOVERY, (ulong *)&ap->status );
			hfc_mlpf_mck_recovery_glpar(ap);
			hfc_w_stop(ap, HFC_MLPF_MCKEND_TMR) ;
			hfc_w_start(ap, HFC_MLPF_MCKEND_TMR) ;
			
			return;
		}
		// mck rec INT
		else if( !(hyp_status & HFC_HG_HYPSTATUS_MCK ) && 
				!(hyp_status & HFC_HG_HYPSTATUS_FCSTP) &&
				!(hyp_status & HFC_HG_HYPSTATUS_FMCK ) )
		{
			hfc_w_stop(ap, HFC_MLPF_MCKEND_TMR) ;
			clear_bit(HFC_MLPF_WAIT_MCKEND, (ulong *)&ap->status );
			hfc_mlpf_mckend_int_glpar(ap);
			
			return;
		}
		// check Force CSTP INT
		else if( hyp_status & HFC_HG_HYPSTATUS_FCSTP)
		{
			hfc_mlpf_change_state(ap, HFC_HG_HYPSTATUS_FCSTP, HFC_DISABLE_HYPER_STATE );
			
			//hfc_write_reg(ap, ( uint )HFC_IOSPACE_INTA_MSK,( char )0x4, HFC_MLPF_HWERR );
			
			return;
		}
		// check FMCK INT
		else if( hyp_status & HFC_HG_HYPSTATUS_FMCK )
		{
			hfc_mlpf_change_state(ap, HFC_HG_HYPSTATUS_FMCK, HFC_DISABLE_HYPER_STATE );
			clear_bit(HFC_MLPF_WAIT_FMCK, (ulong *)&ap->status );
			
			return;
		}
	}
	
	/* other factor should not happen because int mask is disabled */
	hfc_errlog(ap,NULL,NULL,HFC_ERRLOG_TYPE_NONE,ERRID_HFCP_EVNT3,
		0xb1,(uchar*)&hyp_status,4);
	
	HFC_DBGPRT(" hfcldd : hfc_hwerr_int - end\n");
	return;
}


/*
 * Function:    hfc_mlpf_mck_recovery_glpar
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_mlpf_mck_recovery_glpar(
	struct adap_info            *ap)
{
	struct target_info          *target;
	uint                        lun;
	uint                        lp;
	
	HFC_MAILBOX_UNLOCK( ap, HFC_MAILBOX_BUSY); 
	
	for(lp=0 ; lp<MAX_TARGET_PROBE ; lp++)						
	{
		target = hfc_hash_target_info(ap, lp);
		if (target != NULL)
		{
			hfc_cancel_scsi_cmd(
				ap, target, 0, NULL, SCS_MCK, HFC_CSCSI_ERROR,		/* FCLNX-0429 */
				TRUE, TRUE, HFC_FLASH_TARGET);
			target->status = HFC_NON_STATUS ; 
			
			for (lun=0;lun<MAX_DEV_CNT;lun++)
				target->lustat[lun] = 0x00;
			
			if ( hfc_manage_info.hfcldd_mp_mod ) {
				hfc_manage_info.npubp->hfc_forced_offline_e(target, TRUE);
			}
		}
	}
	
	ap->mp_adap_info->mck_err_cnt++ ;	/* FCLNX-GPL-057 */
	
	hfc_write_reg(ap, ( uint )HFC_IOSPACE_INTA_MSK,( char )0x4, ( int )HFC_MLPF_HWERR );
	hfc_reset_watchdog(ap);
	
	return;
}


/*
 * Function:    hfc_mlpf_mckend_int
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_mlpf_mckend_int(
	struct adap_info            *ap)
{
	hfc_w_stop(ap, HFC_MLPF_MCKEND_TMR) ;
	clear_bit(HFC_MLPF_WAIT_MCKEND, (ulong *)&ap->status );
	
	set_bit(HFC_WAIT_LINKUP, (ulong *)&ap->status);
	hfc_w_stop( ap, HFC_LINKUP2_TMR );
	hfc_w_start( ap, HFC_LINKUP2_TMR );
}


/*
 * Function:    hfc_mlpf_mckend_int_glpar
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_mlpf_mckend_int_glpar(
	struct adap_info            *ap)
{
	hfc_mlpf_errlog_glpar(ap);
	
	hfc_reset_adap_info(ap);
	/* WS Comu Area(0x318) */
	hfc_reset_start(ap, HFC_SET_WS80);
	/* INITTBL ADR(0x310)  */
	/* ALPA(0x319)         */
	hfc_reset_start(ap, HFC_SET_INIADR);
	
	hfc_reset_start(ap, HFC_FW_START); 
	clear_bit( HFC_MCK_RECOVERY, (ulong *)&ap->status );
	
	if( test_bit( HFC_NEED_LINK_INIT, (ulong *)&ap->status)){
		if(ap->mck_on_sleep) {
			hfc_wake_up(&ap->mck_event, &ap->mck_event_wait);
		}
	} 
	
	/* Interruption mask setting */
	hfc_write_reg(ap,( uint )HFC_IOSPACE_INTA_MSK,( char )0x4, HFC_INTA_MASK_MLPF);
	
	/* Link Up Timer Set */
	//@MLPF
	set_bit(HFC_WAIT_LINKUP, (ulong *)&ap->status);
	
	hfc_w_stop( ap, HFC_LINKUP2_TMR );
	hfc_w_start( ap, HFC_LINKUP2_TMR );
	
	if( test_bit(HFC_MB_PROL, (ulong *)&ap->mb_status) )
	{
		hfc_wake_up(&ap->mb_event, &ap->mb_event_wait);
	}
	
	return;
}


/*
 * Function:    hfc_mlpf_fcstp_int
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *  type       - 
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_mlpf_fcstp_int(
	struct adap_info            *ap,
	uchar                       type)
{
	
	hfc_chk_stop( ap, TRUE ) ;

	if(test_bit(HFC_MB_PROL, (ulong *)&ap->mb_status )) 
	{
		hfc_wake_up(&ap->mb_event, &ap->mb_event_wait);
	}
	
	if( (test_bit( HFC_WAIT_LINK_INIT, (ulong *)&ap->status)) || (ap->initialize == 1) )
	{
		unlock_mailbox( ap ) ;
		hfc_wake_up(&ap->init_event,&ap->int_a_poll);
		clear_bit( HFC_WAIT_LINK_INIT, (ulong *)&ap->status);
	}
	
	return;
}


/*
 * Function:    hfc_mlpf_issue_ffcstp
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *  type       - 
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_mlpf_issue_ffcstp(
	struct adap_info            *ap,
	uchar                       type)
{
	uint                        int_a_reg;
	
	int_a_reg = (uint)hfc_read_reg(ap, (uint)HFC_IOSPACE_INTA, (char)0x4);
	
	if( HFC_MMODE_CHECK_SHADOW(ap) )
	{
		if( ( int_a_reg & HFC_MLPF_HWERR ) &&
			( hfc_mlpf_check_state(ap, HFC_HG_HYPSTATUS_FCSTP | HFC_HG_HYPSTATUS_FCSTP_IML, HFC_CHECK_HYPER_STATE ) ) )
			return;
	}
	else
	{//Guest LPAR nothing is done and return is done
		return;
	}
	
	if ( !test_bit (HFC_CHK_STOP, (ulong *)&ap->status ) )
	{
		set_bit( HFC_MLPF_WAIT_FCSTP, (ulong *)&ap->status );
		
		hfc_w_stop(ap, HFC_MLPF_FCSTP_TMR) ;
		hfc_w_start(ap, HFC_MLPF_FCSTP_TMR) ;
		
		hfc_write_reg(ap, ( uint )HFC_IOSPACE_INTA_MSK,( char )0x4, HFC_MLPF_HWERR );
		
		if( type == HFC_ABEND_FCSTP )
			hfc_write_hg_reg(ap, HFC_IOHGSPC_CMNDREG, 4, HFC_MLPF_FFCSTP);
		else if ( type == HFC_ABEND_FCSTP_IML )
			hfc_write_hg_reg(ap, HFC_IOHGSPC_CMNDREG, 4, HFC_MLPF_FFCSTP_IML);
	}
	
	return;
}


/*
 * Function:    hfc_mlpf_issue_ffmck
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *  type       - 
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_mlpf_issue_ffmck(
	struct adap_info            *ap,
	uchar                       type)
{
	struct mp_adap_info         *mpap = ap->mp_adap_info;
	uint                        int_a_reg;
	uint                        wk_reg;
	
	int_a_reg = (uint)hfc_read_reg(ap, (uint)HFC_IOSPACE_INTA, (char)0x4);
	
	if( HFC_MMODE_CHECK_SHADOW(ap) )
	{
		if( ( int_a_reg & HFC_HWERR_MCK ) ||
			( ( int_a_reg & HFC_MLPF_HWERR ) &&
			( hfc_mlpf_check_state(ap, HFC_HG_HYPSTATUS_FMCK, HFC_CHECK_HYPER_STATE ) ) ) )
			return;
	}
	else
	{   //Guest LPAR
		if( ( (int_a_reg & HFC_MLPF_HWERR ) &&
			( hfc_mlpf_check_state(ap, HFC_HG_HYPSTATUS_ENABLE | HFC_HG_HYPSTATUS_MCK, HFC_CHECK_HYPER_STATE ) ) ) ||
			( ( int_a_reg & HFC_MLPF_HWERR ) &&
			( hfc_mlpf_check_state(ap, HFC_HG_HYPSTATUS_FMCK, HFC_CHECK_HYPER_STATE ) ) ) )
			return;
	}
	
	if( !test_bit( HFC_MLPF_WAIT_FMCK, (ulong *)&ap->status ) &&
		!test_bit( HFC_WAIT_T3, (ulong *)&ap->status ) &&
		!test_bit( HFC_HWCHKSTOP, (ulong *)&mpap->status ) )
	{
		set_bit( HFC_MLPF_WAIT_FMCK, (ulong *)&ap->status );

		
		if( HFC_MMODE_CHECK_SHADOW(ap) )
		{   // shadow LPAR only start FMCK TMR
			hfc_w_stop(ap, HFC_MLPF_FMCK_TMR) ;
			hfc_w_start(ap, HFC_MLPF_FMCK_TMR) ;
		}
		else
		{   // Guest LPAR set HFC_WAIT_T3
			set_bit( HFC_WAIT_T3, (ulong *)&ap->status );
		}
		
		wk_reg = 0;                                                             /* FCLNX-0389 */
		wk_reg |= type & 0xff;
		wk_reg <<= 16;
		wk_reg |= ap->mb->mb_init.command & 0xff;
		wk_reg <<= 8;
		wk_reg |= ap->mb->mb_init.sub_cmd & 0xff;
		hfc_write_reg(ap,( uint )HFC_IOSPACE_DRV_USED0,( char )0x4, wk_reg);    /* FCLNX-0389 */
		
		hfc_write_hg_reg(ap, HFC_IOHGSPC_CMNDREG, 4, HFC_MLPF_FFMCK);
	}
	
	return;
}


/*
 * Function:    hfc_mlpf_cstpend_int
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_mlpf_cstpend_int(
	struct adap_info            *ap)
{
	// Guest LPAR only
	uint                        lp;
	struct target_info			*target;
	
	hfc_mlpf_errlog_glpar(ap);
	
	for(lp=0 ; lp<MAX_TARGET_PROBE ; lp++)
	{
		target = hfc_hash_target_info(ap, lp);
		if( target != NULL )
		{
			hfc_cancel_scsi_cmd(ap,target,0, NULL, SCS_MCK, HFC_CSCSI_ERROR,	/* FCLNX-0429 */
					TRUE, TRUE, HFC_FLASH_TARGET);
			target->status = HFC_NON_STATUS;
			
			if ( hfc_manage_info.hfcldd_mp_mod ) {
				hfc_manage_info.npubp->hfc_forced_offline_e(target, TRUE);
			}
		}
	}
	
	clear_bit(HFC_MCK_RECOVERY, (ulong *)&ap->status);
	if( test_bit( HFC_NEED_LINK_INIT, (ulong *)&ap->status)){
		if(ap->mck_on_sleep) {
			hfc_wake_up(&ap->mck_event, &ap->mck_event_wait);	
		}
	} 

	set_bit(HFC_CHK_STOP, (ulong *)&ap->status );
	
	hfc_write_reg(ap, ( uint )HFC_IOSPACE_INTA_MSK,( char )0x4, ( int )0x00000000 );
	clear_bit(HFC_ONLINE, (ulong *)&ap->status );
	
	return;
}


/*
 * Function:    hfc_mlpf_errlog_slpar
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_mlpf_errlog_slpar(
	struct adap_info        *ap)
{
	struct mp_adap_info     *mpap;
	int                     rc;
	
	mpap = ap->mp_adap_info;
	
	rc = lock_try_mpap( mpap );	
	
	if( rc == 0 ){		/* fail to aquire lock */
		set_bit( HFC_LOCK_WAIT_6, (ulong *)&ap->mpap_lock );
		hfc_w_stop( ap, HFC_MPAP_LOCK_TMR );
		hfc_w_start( ap, HFC_MPAP_LOCK_TMR );
		return;
	}
	
	if( test_bit( HFC_LOCK_WAIT_6, (ulong *)&ap->mpap_lock ) )
	{
		/* stop retry timer */
		clear_bit( HFC_LOCK_WAIT_6, (ulong *)&ap->mpap_lock );
		hfc_w_stop( ap, HFC_MPAP_LOCK_TMR );
	}
	
	if ( ! hfc_mlpf_check_state(ap, HFC_HG_HYPSTATUS_MCKLOG, HFC_CHECK_HYPER_STATE ) )
	{
		hfc_mlpf_change_state(ap, HFC_HG_HYPSTATUS_MCKLOG,HFC_ENABLE_HYPER_STATE );
		
		hfc_mlpf_indacc(ap);
		
		hfc_mlpf_change_state(ap, HFC_HG_HYPSTATUS_MCKLOG,HFC_DISABLE_HYPER_STATE );
	}
	
	HFC_ADAP_UNLOCK(mpap,HFC_MP_ADAP_BUSY);
}


/*
 * Function:    hfc_mlpf_errlog_glpar
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *
 * Returns:     
 *
 * Notes:       
 */
void hfc_mlpf_errlog_glpar(
	struct adap_info        *ap)
{
	struct mp_adap_info     *mpap;
	struct hfc_err_rec		*err_rec;	/* FCLNX_GPL-0151 */
	uchar                   data[16];
	int                     rc;
	int                     err_no=0;
	
	mpap = ap->mp_adap_info;
	
	rc = lock_try_mpap( mpap );	
	
	if( rc == 0 ){		/* fail to aquire lock */
		set_bit( HFC_LOCK_WAIT_6, (ulong *)&ap->mpap_lock );
		hfc_w_stop( ap, HFC_MPAP_LOCK_TMR );
		hfc_w_start( ap, HFC_MPAP_LOCK_TMR );
		return;
	}
	
	if( test_bit( HFC_LOCK_WAIT_6, (ulong *)&ap->mpap_lock ) )
	{
		/* stop retry timer */
		clear_bit( HFC_LOCK_WAIT_6, (ulong *)&ap->mpap_lock );
		hfc_w_stop( ap, HFC_MPAP_LOCK_TMR );
	}
	
	hfc_mlpf_indacc(ap);
	
//	memset(&err_rec, 0, sizeof(struct hfc_err_rec));			/* FCLNX_GPL-0151 */
//	HFC_MEMCPY( (uchar *)&err_rec, ap->mlpf_drv_log, 0x40);		/* FCLNX_GPL-0151 */
	err_rec = (struct hfc_err_rec *) ap->mlpf_drv_log;			/* FCLNX_GPL-0151 */
	
	HFC_4L_TO_4B(err_no, err_rec->head.err_num);				/* FCLNX_GPL-0151 */
	HFC_MEMCPY( data, err_rec->hfc_drv_log, 16);				/* FCLNX_GPL-0151 */
	
	if( ( err_no == 0x00000035 ) ||
		( err_no == 0x00000036 ) )
	{
		hfc_errlog(
			ap,NULL,NULL,HFC_ERRLOG_TYPE_IMLLOG, ERRID_HFCP_ERRF, err_no, data,16) ;
	}
	else if ( ( err_no == 0x00000077 ) ||
		( err_no == 0x00000078 ) )
	{
		hfc_errlog(
			ap,NULL,NULL,HFC_ERRLOG_TYPE_IMLLOG, ERRID_HFCP_ERRC, err_no, data,16) ;
	}
	else if ( ( err_no == 0x00000071 ) ||
		( err_no == 0x00000031 ) )
	{
		hfc_errlog(
			ap,NULL,NULL,HFC_ERRLOG_TYPE_CHKSTP,ERRID_HFCP_ERR1, err_no,data,16) ;
	}
	else if( err_no == 0x0000002d )
	{
		hfc_errlog(
			ap,NULL,NULL,HFC_ERRLOG_TYPE_MCK,ERRID_HFCP_ERR2, err_no, data,16) ;
	}
	else{
		hfc_errlog(
			ap,NULL,NULL,HFC_ERRLOG_TYPE_MCK,ERRID_HFCP_ERR4, err_no,data,16) ;
	}
	
	HFC_ADAP_UNLOCK(mpap, HFC_MP_ADAP_BUSY);
}


/*
 * Function:    hfc_mlpf_indacc
 *
 * Purpose:     
 *
 * Arguments:   
 *  ap         - pointer to adap_info
 *
 * Returns:     
 *
 * Notes:       
 */
uint hfc_mlpf_indacc(
	struct adap_info        *ap)
{
	uint                    flag;
	uchar                   *log_area;
	struct mp_adap_info     *mpap;
	uint                    reg_addr;
	
	mpap = ap->mp_adap_info;
	
	log_area = (uchar *)mpap->hw_log;
	
	flag = hfc_read_hg_reg(ap, HFC_IOHGSPC_INDACC0, 4);
	if ( flag & HFC_MLPF_INDACC_FLG )
	{
		HFC_ADAP_UNLOCK(mpap,HFC_MP_ADAP_BUSY);
		return HFC_MLPF_INDACC_DISABLE;
	}
	
	hfc_write_hg_reg(ap, HFC_IOHGSPC_CMNDREG, 4, HFC_MLPF_SET_INDACC);
	
	/* It is repeated Read as for the right or wrong of INDACC use flag */
	flag = (uint) hfc_read_hg_reg(ap, HFC_IOHGSPC_INDACC0, 0x4);
	if ( !(flag & HFC_MLPF_INDACC_FLG) )
	{
		HFC_ADAP_UNLOCK(mpap,HFC_MP_ADAP_BUSY);
		return HFC_MLPF_INDACC_USED;
	}
	
	for ( reg_addr=0; reg_addr<HFC_HWLOG_SIZE; reg_addr+=0x80)
	{
		hfc_write_hg_reg(ap, HFC_IOHGSPC_INDACC3, 4, reg_addr);
		
		if ( HFC_MMODE_CHECK_SHADOW (ap) )
		{
			if ( reg_addr == 0 )
				hfc_mlpf_set_mmio_hg(ap, (uchar *)ap->mlpf_drv_log, HFC_IOHGSPC_INDAREA, 0x40 );
			else
				hfc_mlpf_set_mmio_hg(ap, (uchar *)&log_area[reg_addr], HFC_IOHGSPC_INDAREA, 0x80);
		}
		else if ( HFC_MMODE_CHECK_SHARED(ap) && ( !HFC_MMODE_CHECK_SHADOW(ap) ) )
		{
			if ( reg_addr == 0 )
				{
					memset(ap->mlpf_drv_log, 0, sizeof(ap->mlpf_drv_log));
					hfc_mlpf_get_mmio_hg(ap, (uchar *)ap->mlpf_drv_log, HFC_IOHGSPC_INDAREA, 0x40 );
				}
			else
				hfc_mlpf_get_mmio_hg(ap, (uchar *)&log_area[reg_addr], HFC_IOHGSPC_INDAREA, 0x80);
		}
	}
	
	hfc_write_hg_reg(ap, HFC_IOHGSPC_CMNDREG, 4, HFC_MLPF_RESET_INDACC);
	
	HFC_ADAP_UNLOCK(mpap,HFC_MP_ADAP_BUSY);	
	return HFC_MLPF_INDACC_SUCCESS;
}

