/*
 * hfcl_stra_trace.h
 * Copyright (C) 2007, Hitachi, Ltd.
 * Author(s): Yoshihiro Toyohara <yoshihiro.toyohara.qs@hitachi.com>
 */
/*
 * $Id: hfcl_stra_trace.h,v 1.7.8.7.2.1.6.1 2009/01/20 09:31:36 eteshima Exp $
 */


/*------ trace sub_id ---------------*/

#define HFC_TRC_BUILDIO         0x71
#define HFC_TRC_STARTIO         0x72
#define HFC_TRC_SRB_SET         0x73
#define HFC_TRC_ABORT_CMD       0x75
#define HFC_TRC_ABORT           0x76
#define HFC_TRC_RESET           0x77
#define HFC_TRC_BUS_RESET       0x78
#define HFC_TRC_SHUTDOWN        0x79
#define HFC_TRC_FLUSH           0x7a
#define HFC_TRC_ISSUE_TMGM      0x7b
#define HFC_TRC_RESETBUS        0x7c
#define HFC_TRC_IODONE          0x7f

#define	HFC_TRC_STRATEGY        0x81
#define	HFC_TRC_START           0x82
#define	HFC_TRC_CAN_XOB         0x83
#define	HFC_TRC_CAN_WE          0x84
#define HFC_TRC_GET_IOV         0x85
#define	HFC_TRC_RES_CHK         0x87
#define	HFC_TRC_IOVUP           0x88
#define	HFC_TRC_DMA_MAP         0x89
#define	HFC_TRC_MK_CMD          0x8a
#define	HFC_TRC_ENQ_XOB         0x8b
#define HFC_TRC_FRAME_CHK       0x8c
#define	HFC_TRC_CAN_WX			0x8d
#define	HFC_TRC_CLR_WR			0x8e

#define HFC_TRC_RST_NF			0x98

#define HFC_TRC_ENQ_WX			0xcC
#define HFC_TRC_RST_CBACK		0xcD
#define HFC_TRC_RST_FREE		0xcE

/*--------------------------------------*/
struct trc_com {
	uchar			id ;
	uchar			sub_id ;
	uchar			cmnd ;
	uchar			timeout ;
	uchar			rsv2 ;							/* FCLNX-0659 */
	uchar			target_id ;						/* FCLNX-0659 */
	uchar			lun_id ;
	uchar			use_sg ;
	uint			adap_status ;
	uint			target_status ;
	uchar			target_flags ;
	uchar			target_pseq ;
	ushort			target_dev_flags ;
	uchar			lunstat ;
	uchar			open_status ;
	ushort			rsv1 ;
};

/*---- trace format --------------------*/
struct stra_trc1 {
	struct trc_com		com ;

	uint			a_scsi_id ;						/* adaptor scsi_id */
	uint			d_scsi_id ;						/* target  scsi_id */
	ushort			wx_que_cnt ;					/* Scsi start waiting count */
	ushort			we_que_cnt ;					/* Scsi response waiting count */
	uchar			cmd_status ;					/* Hfc_pkt status information */
	uchar			rsv1[2] ;														/* FCLNX-0659 */
	uchar			cmd_xob ;						/* Hfc_pkt allocation xob# */	/* FCLNX-0659 */
	uint			cmd_timeout ;					/* Hfc_pkt timeout time */
	uint			cmd_flags ;						/* Hfc_pkt control flag  */
	uint			cmd_iov_no ;					/* hfc_pkt - seg_info# */
	uint			cmd_iov_cnt ;					/* hfc_pkt - Number of seg_info */
	uint			adap_status ;					/* hfc_pkt adap_status */
	uint			pkt_flag ;						/* scsi_pkt flag byte */
	uint			pkt_time ;						/* scsi_pkt Time setting value */
	uint			pkt_resid ;						/* scsi_pkt The remainder byte */
	uint			pkt_state ;						/* scsi_pkt state */
	uint			serial_number ;					/* scsi_pkt statistics */
	uint			pkt_result ;					/* scsi_pkt result */
	uint64_t		scsi_exec_cnt ;					/* Number of scsi start demands */
	uint64_t		scsi_end_cnt ;					/* Number of scsi responses */
	uint64_t		scsi_err_cnt ;					/* Number of abnormal terminations of scsi */
};

struct stra_trc2 {
	struct trc_com		com ;

	uint			d_scsi_id ;						/* target  scsi_id */
	ushort			wx_que_cnt ;					/* Scsi start waiting count */
	ushort			we_que_cnt ;					/* Scsi response waiting count */
	ushort			xob_no ;
	uchar			xob_exec_cnt ;
	uchar			xob_wait_exec_cnt ;
	uint			iov_no ;
	uint			iov_map_cnt ;
	uint			save_xob_outp ;
	uint			xob_outp_end;
	uint			xob_outp;
	uint			xob_inp;
	uint64_t		new_login_need ;
	uchar			resv1[6];						/* FCLNX-0659 */
	ushort			next_dstart_cnt ;				/* FCLNX-0659 */
	uint			frame_chkp ;
	uint			frame_inp ;
	uchar			cmd_status ;					/* Hfc_pkt status information */
	uchar			cmnd ;
	uchar			rsv2;							/* FCLNX-0659 */
	uchar			cmd_xob ;						/* Hfc_pkt allocation xob# */ /* FCLNX-0659 */
	uint			cmd_timeout ;					/* Hfc_pkt timeout time    */
	uint			cmd_flags ;						/* Hfc_pkt control flag    */
	uint			cmd_iov_no ;					/* hfc_pkt - seg_info#     */
	uint			cmd_iov_cnt ;					/* hfc_pkt - Number of seg_info */
	uint			cmd_data_size;					/* Forwarding data size    */
	uint			adap_status ;					/* hfc_pkt adap_status     */
};

struct stra_trc3 {
	struct trc_com	com ;

	uint			a_scsi_id ;						/* adaptor scsi_id */
	uint			d_scsi_id ;						/* target  scsi_id */
	uint			xob_outp ;
	uint			xob_inp ;
	uchar			type ;							/* cancel type 	*/
	uchar			lun ;							/* lun#		 	*/
	ushort			rsv1;
	uint			adap_status;					/* adap_status(argument) */
	uint			we_que_cnt;
	uint			wx_que_cnt;
};

struct stra_trc4 {
	struct trc_com	com ;

	uint			a_scsi_id ;						/* adaptor scsi_id */
	uint			xob_outp ;
	uint			xob_inp ;
	uchar			xob_exec_cnt ;
	uchar			xob_wait_exec_cnt ;
	ushort			rsv0;
	ushort			wx_que_cnt ;					/* Scsi start waiting count */
	ushort			we_que_cnt ;					/* Scsi response waiting count */
	uint			iov_no ;
	uint			iov_map_cnt ;
	uint			save_xob_outp ;
	uint			xob_outp_end;
	uint			frame_chkp ;
	uint			frame_inp ;
	uint			hfcp_iov_no ;
	uint			hfcp_iov_cnt ;
	uint			cmd_flags ;						/* Hfc_pkt control flag  */
	uint			cmd_dmacount ;					/* Data transfer length  */
	uint			cmd_cookiecnt ;					/* cookie count          */
	ushort			xob_no ;						/* Hfc_pkt allocation xob# */
	uchar			rsv1;
	uchar			type ;							/* 0:alloc,1:cancel      */
	uint			pos ;							/* bit position          */
	uint			cnt ;							/* bit count             */
};

struct stra_trc5 {
	struct trc_com	com ;

	uint			d_scsi_id ;						/* target  scsi_id */
	uint			xob_outp ;
	ushort			xob_no ;
	uchar			xob_exec_cnt ;
	uchar			xob_wait_exec_cnt ;
	ushort			wx_que_cnt ;					/* Scsi start waiting count */
	ushort			we_que_cnt ;					/* Scsi response waiting count */
	uint			fcp_cntl ;
	uint			fcp_dl ;
	uchar			cdb[16] ;
	uchar			xob[48] ;
};

struct stra_trc6 {
	struct trc_com	com ;

	uint			cmd_flags ;						/* Hfc_pkt control flag  */
	ushort			cmd_cdblen ;
	ushort			cmd_scblen ;
	uint			adap_status ;
	uint			cmd_cookie ;
	uint			cmd_ncookie ;
	uint			cmd_cookiecnt ;
	uint			cmd_nwin ;
	uint			cmd_curwin ;
	uint64_t		cmd_dma_offset ;
	uint64_t		cmd_dma_len ;
	uint64_t		cmd_timeout ;					/* Hfc_pkt timeout time */
	uint			cmd_iov_no ;					/* hfc_pkt - seg_info#	*/
	uint			cmd_iov_cnt ;					/* hfc_pkt - Number of seg_info */
	ushort			cmd_xob ;						/* Hfc_pkt allocation xob# */
	uchar			cmd_status ;					/* Hfc_pkt status information */
	uchar			rsv1 ;
	uint			cmd_dmacount ;					/* Data transfer length		   */
	uint64_t		cmd_forw ;
	uint64_t		cmd_prev ;
	uint			pkt_resid ;						/* scsi_pkt remainder byte */
};

struct stra_trc7 {
	struct trc_com	com ;

	uint			cap ;
	uint			whom ;
	uint			rtn ;
	uint			dma_max ;
	uint			burst_size ;
	uint			scsi_options ;
	uint			linkup_tmo ;
	uint			scsi_reset_deley ;
	uint			need_prop_update ;
	uint			target_scsi_options ;
	uint			cap_info ;
};

/*dmp*/ /*32bitonly*/
struct stra_dump_srb {
	uchar                   id;
	uchar                   sub_id;
	uchar                   resv1[6];
	uint64_t                srb_address;
	uchar                   srb_dump[96];
	uchar                   resv2[8];
};

struct stra_dump_hfcp {
	uchar                   id;
	uchar                   sub_id;
	uchar                   resv1[6];
	uint64_t                hfcp_address;
	uchar                   hfcp_dump[96];
	uchar                   resv2[8];
};

struct stra_dump_weque {
	uchar                   id;
	uchar                   sub_id;
	uchar                   target_id;
	uchar                   hash;
	uint                    we_que_cnt;
	uint                    dmp_pkt[28];
};

